//------------------------------------------------------------------
//
//
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers  8
#property indicator_minimum  -1
#property indicator_maximum 101
#property strict
//
//
//
//
//

enum enPrices
{
   pr_close,      // Close
   pr_open,       // Open
   pr_high,       // High
   pr_low,        // Low
   pr_median,     // Median
   pr_typical,    // Typical
   pr_weighted,   // Weighted
   pr_average,    // Average (high+low+open+close)/4
   pr_medianb,    // Average median body (open+close)/2
   pr_tbiased,    // Trend biased price
   pr_highlow,    // High/low
   pr_tbiased2,   // Trend biased (extreme) price
   pr_haclose,    // Heiken ashi close
   pr_haopen ,    // Heiken ashi open
   pr_hahigh,     // Heiken ashi high
   pr_halow,      // Heiken ashi low
   pr_hamedian,   // Heiken ashi median
   pr_hatypical,  // Heiken ashi typical
   pr_haweighted, // Heiken ashi weighted
   pr_haaverage,  // Heiken ashi average
   pr_hamedianb,  // Heiken ashi median body
   pr_hatbiased,  // Heiken ashi trend biased price
   pr_hatbiased2, // Heiken ashi trend biased (extreme) price
   pr_hahighlow   // Heiken ashi high/low
};

enum enRsiTypes
{
   rsi_rsi,  // Regular RSI
   rsi_wil,  // Slow RSI
   rsi_rap,  // Rapid RSI
   rsi_har,  // Harris RSI
   rsi_rsx,  // RSX
   rsi_cut   // Cuttlers RSI
};

enum enMaTypes
{
   ma_sma,    // Simple moving average
   ma_ema,    // Exponential moving average
   ma_smma,   // Smoothed MA
   ma_lwma,   // Linear weighted MA
   ma_tema    // Triple exponential moving average - TEMA
};

enum enColorOn
{
   cc_onSlope,   // Change color on slope change
   cc_onMiddle,  // Change color on middle line cross
   cc_onLevels   // Change color on outer levels cross
};

extern ENUM_TIMEFRAMES TimeFrameCur        = PERIOD_CURRENT; //Time frame for current
extern ENUM_TIMEFRAMES TimeFrameHtf        = PERIOD_H1;      //Time frame for "higher TF filtering"

extern int        RSIPeriod   = 14;             // RSI period
extern enRsiTypes RsiMethod   = rsi_rsx;        // Rsi type
extern enPrices   Price       = pr_hatbiased2;  // RSI applied to price
extern int        StoPeriod1  = 55;             // Stochastic period 1 (less than 2 - no stochastic)
extern int        StoPeriod2  = 7;              // Stochastic period 2 (less than 2 - no stochastic)
extern int        EMAPeriod   = 15;             // Smoothing period (less than 2 - no smoothing)
extern int        flLookBack  = 25;             // Floating levels look back period
extern double     flLevelUp   = 90;             // Floating levels up level %
extern double     flLevelDown = 10;             // Floating levels down level %

extern int             levupBuffer         = 0;              //"Level Up" buffer number
extern int             levmiBuffer         = 1;              //"Level Middle" buffer number
extern int             levdnBuffer         = 2;              //"Level Down" buffer number
extern int             ValueBuffer         = 3;              //"Value" buffer number
extern int             SlopeBuffer         = 10;             //"Trend" buffer number
extern color           colorUp             = clrLime;
extern color           colorDown           = clrRed;
extern color           colorNeutral        = clrDimGray;
extern int             linesWidth          = 2;
extern string          IndicatorName       = "Double Stochastic RSI mtf"; // Indicator name to use
extern bool            arrowsVisible       = true;           // Arrows visible?
extern bool            arrowsOnNewest      = true;           // Arrows drawn on newst bar of higher time frame bar?
extern string          arrowsIdentifier    = "2ind DSRSI Arrows1"; // Unique ID for arrows
extern double          arrowsUpperGap      = 1.0;              // Upper arrow gap
extern double          arrowsLowerGap      = 1.0;              // Lower arrow gap
extern color           arrowsUpColor       = clrLime;        // Up arrow color
extern color           arrowsDnColor       = clrRed;         // Down arrow color
extern int             arrowsUpCode        = 241;            // Up arrow code
extern int             arrowsDnCode        = 242;            // Down arrow code
extern int             arrowsSize          = 0;              // Arrows size


bool returnBars;
string indicatorFileName;

double levup[],levmi[],levdn[],Value[],ValueUa[],ValueUb[],ValueDa[],ValueDb[],slope[],slopeHtf[],count[];

#define _bufferDefaultCall(_timeFrame,_buff,_y) iCustom(NULL,0,IndicatorName,_timeFrame,RSIPeriod,RsiMethod,Price,StoPeriod1,StoPeriod2,EMAPeriod,flLookBack,flLevelUp,flLevelDown,_buff,_y)


//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//
int OnInit() 
{
   IndicatorBuffers(11);
   SetIndexBuffer(0,levup);    SetIndexStyle(0,DRAW_LINE,STYLE_DOT,EMPTY,clrDimGray);
   SetIndexBuffer(1,levmi);    SetIndexStyle(1,DRAW_LINE,STYLE_DOT,EMPTY,clrDimGray);
   SetIndexBuffer(2,levdn);    SetIndexStyle(2,DRAW_LINE,STYLE_DOT,EMPTY,clrDimGray);
   SetIndexBuffer(3, Value);   SetIndexStyle(3,EMPTY,EMPTY,linesWidth,colorNeutral);
   SetIndexBuffer(4, ValueUa); SetIndexStyle(4,EMPTY,EMPTY,linesWidth,colorUp);
   SetIndexBuffer(5, ValueUb); SetIndexStyle(5,EMPTY,EMPTY,linesWidth,colorUp);
   SetIndexBuffer(6, ValueDa); SetIndexStyle(6,EMPTY,EMPTY,linesWidth,colorDown);
   SetIndexBuffer(7, ValueDb); SetIndexStyle(7,EMPTY,EMPTY,linesWidth,colorDown);
   SetIndexBuffer(8, slope);
   SetIndexBuffer(9, slopeHtf);
   SetIndexBuffer(10, count);
   indicatorFileName = WindowExpertName();
   TimeFrameCur = MathMax(TimeFrameCur,_Period); 
   IndicatorShortName(timeFrameToString(TimeFrameCur)+"/"+timeFrameToString(TimeFrameHtf)+" 2 TF Double Stochastic RSI");

   return(INIT_SUCCEEDED);
}
//---+
void OnDeinit(const int reason)
{ 
   if (!returnBars)
      switch(reason)
      {
         case REASON_PARAMETERS  : ObjectsDeleteAll(ChartID(),arrowsIdentifier,0,OBJ_ARROW);
         case REASON_CHARTCHANGE : ObjectsDeleteAll(ChartID(),arrowsIdentifier,0,OBJ_ARROW);
         case REASON_RECOMPILE   : 
         case REASON_CLOSE       : break;
         default                 : ObjectsDeleteAll(ChartID(),arrowsIdentifier,0,OBJ_ARROW);
      }
}
//---+
//
//
//
//
//


int OnCalculate (const int       rates_total,
                 const int       prev_calculated,
                 const datetime& time[],
                 const double&   open[],
                 const double&   high[],
                 const double&   low[],
                 const double&   close[],
                 const long&     tick_volume[],
                 const long&     volume[],
                 const int&      spread[])
{
   int counted_bars = prev_calculated;
   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
   int limit=MathMax(rates_total-prev_calculated-1,0); count[0]=limit;
    
                  
      if (slope[limit]== 1) CleanPoint(limit,ValueUa,ValueUb);
      if (slope[limit]==-1) CleanPoint(limit,ValueDa,ValueDb);
      int i=limit; for(; i>=0 && !_StopFlag; i--)  
      
//      if (i<Bars-1) 
      {
            if (levupBuffer>=0) levup[i] = _bufferDefaultCall(TimeFrameCur,levupBuffer,i);
            if (levmiBuffer>=0) levmi[i] = _bufferDefaultCall(TimeFrameCur,levmiBuffer,i);
            if (levdnBuffer>=0) levdn[i] = _bufferDefaultCall(TimeFrameCur,levdnBuffer,i);
            
            if (ValueBuffer>=0) Value[i] = _bufferDefaultCall(TimeFrameCur,ValueBuffer,i);
            
            if (SlopeBuffer>=0) { slope[i]    = _bufferDefaultCall(TimeFrameCur,SlopeBuffer,i);
                                  slopeHtf[i] = _bufferDefaultCall(TimeFrameHtf,SlopeBuffer,i); }
         
          ValueDa[i] = EMPTY_VALUE;
          ValueDb[i] = EMPTY_VALUE;
          ValueUa[i] = EMPTY_VALUE;
          ValueUb[i] = EMPTY_VALUE;
   
         
     
             if (slope[i]== 1&&slopeHtf[i] == 1) PlotPoint(i,ValueUa,ValueUb,Value); 
             if (slope[i]==-1&&slopeHtf[i] ==-1) PlotPoint(i,ValueDa,ValueDb,Value);
             
             if (arrowsVisible)
             {
                string lookFor = arrowsIdentifier+":"+(string)Time[i]; ObjectDelete(lookFor);            
                if (i<(Bars-1))
                {
                  if((slope[i] != slope[i+1])||(slopeHtf[i] != slopeHtf[i+1]))

                   {
                      if (slope[i]== 1&&slopeHtf[i] == 1) drawArrow(i,arrowsUpColor,arrowsUpCode,false);
                      if (slope[i]==-1&&slopeHtf[i] ==-1) drawArrow(i,arrowsDnColor,arrowsDnCode, true);
                   }
                }
             }                                     
          
      } 
    
        
   return(rates_total-i-1); 
}
//
//
//
//
//
//
//---+
void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}
//---+
void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i];  first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] =  from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                           second[i] = EMPTY_VALUE; }
}
//---+
string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}
//---+
void drawArrow(int i,color theColor,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+(string)Time[i];
   double gap  = iATR(NULL,0,20,i);   
   
      //
      //
      //
      //
      //

      datetime time = Time[i]; if (arrowsOnNewest) time += _Period*60-1;      
      ObjectCreate(name,OBJ_ARROW,0,time,0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_WIDTH,arrowsSize);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i] + arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i]  - arrowsLowerGap * gap);
}